/**********************************************************************
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.cosmos.rm.validation.internal.launcher;

import java.util.Hashtable;
import java.util.Map;

import org.eclipse.cosmos.rm.validation.internal.artifacts.DocumentNode;
import org.eclipse.cosmos.rm.validation.internal.artifacts.ElementEdge;
import org.eclipse.cosmos.rm.validation.internal.common.ISMLConstants;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

/**
 * This is an abstract implementation of ISMLValidation that will lookup 
 * and invoke the validation class for each SML extension. 
 * 
 * @author Ali Mehregani
 */
public abstract class AbstractSMLValidation implements ISMLValidation
{
	/**
	 * Stores the validation class that is expected to handle an SML 
	 * extension.
	 *
	 * KEY = instance of a class of type ISMLValidation
	 * VALUE = An array of type String indicating the scope for the validation
	 * class.  See ISMLValidation.SCOPE_ for more details  
	 */
	private static Map smlExtensionHandler = new Hashtable();
	
	/**
	 * The scope of this validation class
	 */
	private String[] scope;
	
	static
	{
		loadExtensionHandler();
	}
	
	/**
	 * Load the validation classes required for each scope
	 */
	private static void loadExtensionHandler()
	{
		// TODO: This method needs to be based on an extensions defined in section 1.3 of the
		// design document.  For the first milestone it will be hard-coded.		
		DefaultSMLValidation defaultSMLValidation = new DefaultSMLValidation();		
		defaultSMLValidation.setScope(ISMLValidation.SCOPE_GRP_ALL);
		smlExtensionHandler.put(ISMLValidation.SCOPE_GRP_ALL, defaultSMLValidation);		
	}
	
	public void intialize(Map validationAttribute)
	{
		
	}



	public Object start(Object object)
	{
		// TODO Auto-generated method stub
		return null;
	}
	
	protected Map generateRefGraph()
	{
		return null;
	}
	
	
	/**
	 * @see org.eclipse.cosmos.rm.validation.internal.launcher.ISMLValidation#getScope()
	 */
	public String[] getScope()
	{
		return scope;
	}

	
	/**
	 * @see org.eclipse.cosmos.rm.validation.internal.launcher.ISMLValidation#setScope(java.lang.String[])
	 */
	public void setScope(String[] scope)
	{
		this.scope = scope;
	}

	protected abstract void validate();
	
	protected DefaultHandler getDefaultHandler()
	{
		
		DefaultHandler defaultHandler = new DefaultHandler()
		{
			public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException
			{
				// TODO Auto-generated method stub
				super.startElement(uri, localName, qName, attributes);
			}
		};
		return null;
	}
	
	/**
	 * This default handler is used to generate the directed reference
	 * graph.
	 */
	public static class ReferenceGraphGenerator extends DefaultHandler
	{
		/**
		 * Represents the reference graph
		 * 
		 * KEY = The alias representing a document node
		 * VALUE = An object of type {@link DocumentNode}
		 */
		private Map referenceGraph;
		
		/**
		 * The current node which represents a document 
		 */
		private DocumentNode currentNode;
		
		/**
		 * Set to true when notified of the start of an alias element
		 */
		private boolean aliasElementHit;
		
		/**
		 * Set to true when notified of the start of an uri element
		 */
		private boolean uriElementHit;
		
		/**
		 * Stores the source node of a reference
		 */
		private String srcReferenceNode;
		
		/**
		 * Stores the last element that has been hit
		 */
		private String lastElement;
				
		/**
		 * Current edge of the reference graph node
		 */
		private ElementEdge elementEdge;
		
		/**
		 * The current alias
		 */
		private String currentAlias;
		
		
		/**
		 * @see org.xml.sax.helpers.DefaultHandler#startElement(java.lang.String, java.lang.String, java.lang.String, org.xml.sax.Attributes)
		 */
		public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException
		{
			resetElementStartStatus();
			if (ISMLConstants.DOCUMENT_ELEMENT.equals(localName))
			{							
				currentNode = new DocumentNode();					
			}
			else if (ISMLConstants.ALIAS_ELEMENT.equals(localName))
			{
				aliasElementHit = true;
			}
			else if (ISMLConstants.URI_ELEMENT.equals(localName))
			{
				uriElementHit = true;
				srcReferenceNode = lastElement;
			}
			
			lastElement = localName;
		}

		/**
		 * Resets flags that are set when the handler is notified of the start 
		 * of a specific element. 
		 */
		private void resetElementStartStatus()
		{
			aliasElementHit = false;
			elementEdge = null;
		}

		/**
		 * @see org.xml.sax.helpers.DefaultHandler#characters(char[], int, int)
		 */
		public void characters(char[] ch, int start, int length) throws SAXException
		{
			String textNode = new String(ch, start, length).trim();
			if (currentNode == null || textNode.length() <= 0)
				return;
			
			if (aliasElementHit)
			{
				currentAlias += textNode;				
			}
			else if (uriElementHit)
			{
				if (elementEdge == null)
				{
					elementEdge = new ElementEdge();
					elementEdge.setSourceElement(srcReferenceNode);
				}
				elementEdge.appendToReference(textNode);
			}
		}
		
		/**
		 * @see org.xml.sax.helpers.DefaultHandler#endElement(java.lang.String, java.lang.String, java.lang.String)
		 */
		public void endElement(String uri, String localName, String qName) throws SAXException
		{
			if (ISMLConstants.DOCUMENT_ELEMENT.equals(localName))
			{
				String[] aliases = currentNode.getAliases();
				if (aliases == null)
					return;
				
				for (int i = 0; i < aliases.length; i++)
				{
					referenceGraph.put(aliases[i], currentNode);
				}
			}
			else if (ISMLConstants.ALIAS_ELEMENT.equals(localName))
			{
				currentNode.addAlias(currentAlias);
			}
		}
	}
	
	
	/**
	 * A default concrete SML validation class
	 * 
	 * @author Ali Mehregani
	 */
	public static class DefaultSMLValidation extends AbstractSMLValidation
	{
		public void validate()
		{
			
		}		
	}
}
